function build(varargin)
  p = inputParser; 
  p.addParamValue('verbose', 1, @(x) ismember(x,[0,1]) );
  p.addParamValue('cudapath', '/usr/local/cuda', @ischar );
  p.addParamValue('step', 3, @(x) ismember(x,1:3) );
  p.addParamValue('outpath', '', @ischar );
  p.addParamValue('gccversion', '', @ischar );
  p.addParamValue('debug', false, @islogical );
  p.parse(varargin{:});
  R=p.Results;
  S=checkCUDAconfiguration(R);
  rehash % to update new Thrust.config.m write in checkCUDAconfiguration
  if isempty(R.outpath)
    P=Thrust.tools.getConfigPaths();
    if R.debug
      R.outpath=P.MEXDEBUG_DIR;
    else
      R.outpath=P.MEX_DIR;
    end
  end
  if ~exist(R.outpath,'dir'),mkdir(R.outpath);end
  fprintf('Build in directory :\n -> %s\n',R.outpath)
  if ~isempty(R.gccversion), buildMEXopts(R.gccversion);end
  if R.step>=2, Build_libmThrust(R);end
  if R.step>=3, mThrust_mexcompilation(R,S);end
  clear mex
end


function S=checkCUDAconfiguration(R)
  fprintf('--------------------------------------------\n');
  fprintf('Step [1/3] : mThrust configuration\n')
  fprintf('--------------------------------------------\n');
  toolboxdir=Thrust.tools.getToolboxpath();
  MatlabPath = matlabroot;
  MatlabRelease = ['R',version('-release')];
  CudaPath=R.cudapath;
  if (exist(CudaPath)~=7)
    fprintf('The current CUDA path is not valid:\n  %s\n',CudaPath);
    fprintf('Use \n  Thrust.compile.build(''cudapath'', <Directory> )\n');
    fprintf('to set it. Here <Directory> is a string corresponding to a valid CUDA path\n');
    error('Not a valid CUDA path');
  end
  fprintf('CUDA path: %s\n',CudaPath);
  NVCC=[CudaPath,filesep,'bin',filesep,'nvcc'];
  if (exist(NVCC)~=2)
    error('Not found nvcc compiler');
  end
  fprintf('CUDA compiler [nvcc]: %s\n',NVCC);
  % Compilation test
  filename='getCUDAversion.cpp';
  sourcefile=[toolboxdir,filesep,'build',filesep,'src',filesep,filename];
  if (exist(sourcefile)~=2)
    error('File %s not found',sourcefile);
  end
  command=sprintf('%s %s -o %s',NVCC,sourcefile,'getCUDAversion');
  fprintf('Try to compile %s with nvcc compiler...',filename);
  [statut,result]=system(command);
  if statut~=0
    fprintf('Failed\n');
    fprintf('Compilation failed with command:\n  %s\n',command);
    fprintf('Output:\n%s\n',result);
    error('Compilation Failed');
  end
  fprintf('OK\n');
  fprintf('Try to obtain CUDA version...');
  command='./getCUDAversion';
  [statut,resultat]=system(command);
  resultat=strtrim(strrep(resultat,'\n',''));
  if statut~=0
    fprintf('Xommand Failed:\n  %s\n',command);
    fprintf('Output:\n%s\n',resultat);
    error('Commmand Failed');
  end
  CUDAversion=resultat;
  fprintf(' found %s\n',CUDAversion);
  delete('./getCUDAversion')
  
  filename='getCUDAsms.cpp';
  sourcefile=[toolboxdir,filesep,'build',filesep,'src',filesep,filename];
  if (exist(sourcefile)~=2)
    error('File %s not found',sourcefile);
  end
  command=sprintf('%s %s -o %s',NVCC,sourcefile,'getCUDAsms');
  fprintf('Try to compile %s with nvcc compiler...',filename);
  [statut,result]=system(command);
  if statut~=0
    fprintf('Failed\n');
    fprintf('Compilation failed with command:\n  %s\n',command);
    fprintf('Output:\n%s\n',result);
    error('Compilation Failed');
  end
  fprintf('OK\n');
  fprintf('Try to obtain CUDA version...');
  command='./getCUDAsms';
  [statut,resultat]=system(command);
  resultat=strtrim(strrep(resultat,'\n',''));
  if statut~=0
    fprintf('Failed\n');
    fprintf('Commmand failed:\n  %s\n',command);
    fprintf('Output:\n%s\n',resultat);
    error('Command Failed');
  end
  CUDAsms=resultat;
  fprintf(' found %s\n',CUDAsms);
  delete('./getCUDAsms')
  
  
  fprintf('Check for CUDA Makefile...')
  CUDAmakefile=[toolboxdir,filesep,'build',filesep,'Makefile.cuda-',resultat];
  if (exist(sourcefile)~=2)
    fprintf('Failed\n');
    fprintf('File not found:\n  %s\n',CUDAmakefile);
    error('Not found a Makefile for this CUDA version');
  end
  fprintf('OK\n');
  configfile=[toolboxdir,filesep,'build',filesep,'config'];
  fprintf('Writing config file:\n  %s\n',configfile);
  fid=fopen(configfile,'w');
  fprintf(fid,'CUDA_PATH = %s\n',CudaPath);
  fprintf(fid,'CUDA_VERSION = %s\n',CUDAversion);
  fprintf(fid,'SMS = %s\n',CUDAsms);
  fprintf(fid,'MATLAB_PATH = %s\n',MatlabPath);
  fprintf(fid,'MATLAB_VERSION = %s\n',MatlabRelease);
  if ~isempty(R.gccversion)
    fprintf(fid,'GCC_VERSION = ''%s''\n',R.gccversion);
  end
  fclose(fid);
  configThrust=[toolboxdir,filesep,'+Thrust',filesep,'config.m'];
  fid=fopen(configThrust,'w');
  fprintf(fid,'%% Don''t modify this file\n');
  fprintf(fid,'%% Automaticaly generated by Thrust.build function\n');
  fprintf(fid,'%% Date : %s\n',datestr(now));
  fprintf(fid,'CUDA_PATH = ''%s'';\n',CudaPath);
  fprintf(fid,'CUDA_VERSION = ''%s'';\n',CUDAversion);
  fprintf(fid,'SMS = ''%s'';\n',CUDAsms);
  fprintf(fid,'MATLAB_PATH = ''%s'';\n',MatlabPath);
  fprintf(fid,'MATLAB_VERSION = ''%s'';\n',MatlabRelease);
  if ~isempty(R.gccversion)
    fprintf(fid,'GCC_VERSION = ''%s'';\n',R.gccversion);
  end
  fclose(fid);
  
  S=struct('CUDA_PATH',CudaPath,'CUDA_VERSION',CUDAversion,'MATLAB_PATH',MatlabPath,'MATLAB_VERSION',MatlabRelease,'GCC_VERSION',R.gccversion);
end

function Build_libmThrust(R)
  toolboxdir=Thrust.tools.getToolboxpath();
  fprintf('--------------------------------------------\n');
  fprintf('Step [2/3] : Building mThrust shared library\n')
  fprintf('--------------------------------------------\n');
  fprintf('Create Makefile ...');
  BuildMakefile_libmThrust(toolboxdir,R)
  fprintf('Done\n');
  if R.verbose>0
    fprintf('[Matlab] Build file Makefile.mThrust\n');
    fprintf('[Matlab] Run system command "make -f Makefile.mThrust"\n');
    fprintf('    That can take a few minutes ... be patient\n');
  end
  command='make -f Makefile.mThrust';
  [status,result]=system(command);
  if status~=0
    fprintf('Compilation failed:\n')
    fprintf('  -> %s\n',command);
    fprintf('See Makefile.log\n');
    fid=fopen('Makefile.log','w');
    fprintf(fid,result)
    fclose(fid);
    if R.verbose>0
      %Thrust.extern.cprintf.cprintf('Red','%s',result);
      fprintf('%s\n',result);
    end
  elseif R.verbose>0
    %Thrust.extern.cprintf.cprintf('Blue','%s',result)
    fprintf('%s\n',result);
  end
  %delete('Makefile.mThrust');
end

function BuildMakefile_libmThrust(toolboxdir,R)
  fid = fopen('Makefile.mThrust','w');
  fprintf(fid,'# automaticaly generated by mThrust_mex Matlab function \n');
  fprintf(fid,'MTHRUST_DIR = %s\n',toolboxdir);
  fprintf(fid,'MAIN_DIR = $(MTHRUST_DIR)\n');
  fprintf(fid,'include $(MTHRUST_DIR)/Makefile.inc\n');
  fprintf(fid,'include $(BUILD_DIR)/config\n');
  fprintf(fid,'include $(BUILD_DIR)/Makefile.matlab\n');
  fprintf(fid,'include $(BUILD_DIR)/Makefile.cuda-$(CUDA_VERSION)\n');
  fprintf(fid,'include $(BUILD_DIR)/Makefile.mex\n\n');
  fprintf(fid,'NVCC_FLAGS += -Xcompiler ''-fPIC  -O3 -funroll-loops -msse3'' -D FAST_MATH=1 -D_FORCE_INLINES -O3 -m 64 --use_fast_math $(GENCODE_FLAGS)\n\n');
  fprintf(fid,'INCLUDELIB = -L$(CUDA_PATH)/lib64  -lcudart  $(CUDASDK_LIB)\n\n');
  fprintf(fid,'DEBUGFLAGS =\n');
  if R.debug
    fprintf(fid,'DEBUGFLAGS += -D DEBUG=1\n');
    fprintf(fid,'DEBUGFLAGS += -D VERBOSE=1\n');
    fprintf(fid,'DEBUGFLAGS += -D DEBUG_GPUMEMORY=1\n');
  end
  fprintf(fid,'.SUFFIXES: .$(MEXEXT) .cu\n\n');
  if R.debug
    fprintf(fid,'all : $(MEXDEBUG_DIR)/libmThrust.so\n\n');
    fprintf(fid,'$(MEXDEBUG_DIR)/libmThrust.so : $(SRC_DIR)/mThrust.cu\n');
    fprintf(fid,'\t$(NVCC) $(NVCC_FLAGS) -I$(INC_DIR) -c $(SRC_DIR)/mThrust.cu -o $(MEXDEBUG_DIR)/mThrust.o\n');
    fprintf(fid,'\tg++ -shared -o $(MEXDEBUG_DIR)/libmThrust.so $(MEX_DIR)/mThrust.o\n\n');
    fprintf(fid,'$(MEX_DIR)/mThrust.o : $(MEXSRC)/mThrust.cu\n');
    fprintf(fid,'\t$(NVCC) -c $(MEXSRC)/mThrust.cu $(INCLUDES) $(ALL_CCFLAGS) $(GENCODE_FLAGS) -Xcompiler ''-fPIC  -O3 -funroll-loops -msse3'' -I$(MATLABINC) $(CUDASDK_INC) -o $(MEXDEBUG_DIR)/mThrust.o\n');
  else
    fprintf(fid,'all : $(MEX_DIR)/libmThrust.so\n\n');
    fprintf(fid,'$(MEX_DIR)/libmThrust.so : $(SRC_DIR)/mThrust.cu\n');
    fprintf(fid,'\t$(NVCC) $(NVCC_FLAGS) -I$(INC_DIR) -c $(SRC_DIR)/mThrust.cu -o $(MEX_DIR)/mThrust.o\n');
    fprintf(fid,'\tg++ -shared -o $(MEX_DIR)/libmThrust.so $(MEX_DIR)/mThrust.o\n\n');
    fprintf(fid,'$(MEX_DIR)/mThrust.o : $(MEXSRC)/mThrust.cu\n');
    fprintf(fid,'\t$(NVCC) -c $(MEXSRC)/mThrust.cu $(INCLUDES) $(ALL_CCFLAGS) $(GENCODE_FLAGS) -Xcompiler ''-fPIC  -O3 -funroll-loops -msse3'' -I$(MATLABINC) $(CUDASDK_INC) -o $(MEX_DIR)/mThrust.o\n');
  end
  fclose(fid);
end

function mThrust_mexcompilation(R,S)
  fprintf('--------------------------------------------\n');
  fprintf('Step [3/3] : mex files compilation ( can be very long)\n');
  fprintf('--------------------------------------------\n');
  toolboxdir=Thrust.tools.getToolboxpath();
  
  SrcDir{1}=[toolboxdir,filesep,'build',filesep,'src',filesep];
  Files{1}=dir([SrcDir{1},'mex*.cu']);
  SrcDir{2}=[toolboxdir,filesep,'build',filesep,'auto_src',filesep,'cmathfuncs',filesep];
  Files{2}=dir([SrcDir{2},'mex*.cu']);
  SrcDir{3}=[toolboxdir,filesep,'build',filesep,'auto_src',filesep,'mathfuncs',filesep];
  Files{3}=dir([SrcDir{3},'mex*.cu']);
  
  if isempty(R.outpath)
    P=Thrust.tools.getConfigPaths();
    if R.debug
      R.outpath=P.MEXDEBUG_DIR;
    else
      R.outpath=P.MEX_DIR;
    end
  end
  if ~exist(R.outpath,'dir'),mkdir(R.outpath);end
  if R.verbose>0
    fprintf('Compiled files are stored in :\n   %s\n',R.outpath);
  end
  for i=1:3,  nF(i)=length(Files{i});end
  nT=sum(nF);
  count=1;TotalCpuTime=0;
  for j=1:3
    files=Files{j};
    for i=1:nF(j)
      S1=sprintf('(%d/%d) %s compilation ',count,nT,Files{j}(i).name);
      fprintf('%s',S1);
      tstart=tic;
      Thrust.compile.mex([SrcDir{j},Files{j}(i).name],'outpath',R.outpath,'verbose',0);
      cputime=toc(tstart);
      TotalCpuTime=TotalCpuTime+cputime;
      if cputime>0.5
        S2=sprintf(' %.2f(s)',cputime);
      else
        S2=sprintf(' OK');
      end
      line=repmat('.',1,70-length(S2)-length(S1));
      fprintf('%s%s\n',line,S2);
      count=count+1;
    end
  end
  fprintf('  -> Step [3/3] done in %.2f(s)\n',TotalCpuTime);
end

function buildMEXopts(GCC_VERSION)
  P=Thrust.tools.getConfigPaths();
  mexopts=[matlabroot,'/bin/mexopts.sh'];
  [success,message,MESSAGEID]=copyfile(mexopts,P.BUILD_DIR,'f');
  if ~success
    error(message)
  end
  cmd=sprintf('sed -i "s/CC=''gcc''/CC=''gcc-%s''/g; s/CXX=''g++''/CXX=''g++-%s''/g; s/FC=''gfortran''/FC=''gfortran-%s''/g" %s', ...
         GCC_VERSION,GCC_VERSION,GCC_VERSION,[P.BUILD_DIR,filesep,'mexopts.sh']);
  [status,result]=system(cmd);
  if status~=0
    error(result)
  else
    fprintf('mexopts.sh file created to use GCC %s\n',GCC_VERSION);
  end
end

