function varargout=GeneralizedNewtonFractal(f,fp,z0,varargin);
% NEWTON solves a nonlinear system of equations
%   z=Newton(f,fp,z0,tol,maxiter,fp); solves the nonlinear 
%   system of equations f(z)=0 using Newtons methods with 
%   z real or complex, starting with the initial guess x0 
%   up to a tolerance tol, doing at most maxit iterations. 
%   The analytical derivative of f is given by the 
%   parameter fp. The initial guess z0 could be an array. 
%   (vectorized Matlab version).
  import Thrust.samples.NewtonFractal.GeneralizedNewton
  p = inputParser;
  p.addParamValue('tol', 1e-8, @isscalar );
  p.addParamValue('itermax', 200, @isscalar );
  p.addParamValue('verbose', 0, @(x) ismember(x,0:1) );
  p.addParamValue('a', 1, @isscalar );
  p.parse(varargin{:});
  R=p.Results;
  tstart=tic();
  N=size(z0);
  z=zeros(N);
  count=zeros(N);
  if R.verbose>0, upd = textprogressbar(N(1));end
  for s=1:N(1)
    for r=1:N(2)
      [zc,S]=GeneralizedNewton(f,fp,z0(r,s),'tol',R.tol, 'itermax',R.itermax,'a',R.a);
      z(r,s)=zc;
      count(r,s)=S.iter;
    end
   if R.verbose>0, upd(s);end
  end
  T=toc(tstart);
  if nargout>=1,varargout{1}=z;end
  if nargout>=2,varargout{2}=count;end
  if nargout==3,varargout{3}=T;end
end