clear all
close all
import Thrust.samples.NewtonFractal.*
% Use -singleCompThread option when starting MATLAB to limit MATLAB to a single computational thread

% Don't change functions
f=@(z) z.^3-1;
df=@(z) 3*z.^2;
tol=1e-10;
itermax=300;
a=1;
box=[-2,2,-2,2];


fprintf('0) CUDA/Mex compilation : \n')
P=Thrust.tools.getCurrentFunctionPath();
tstart=tic();
cudamexfile=[P,'mex_GeneralizedNewtonFractal_simple.cu'];
Thrust.compile.mex(cudamexfile,'samepath',true,'verbose',0)
Tcomp=toc(tstart);
fprintf('  -> Compilation in %.3f(s)\n',Tcomp);
tstart=tic();
GeneralizedNewtonFractalGPUsimple(complex(1,1));
Tcomp(2)=toc(tstart);
fprintf('  -> Preload mex file in %.3f(s)\n',Tcomp(2));



LN=1000:500:5000;LN=[LN;LN]';
%LN=1000:500:2000;LN=[LN;LN]';
n=size(LN,1);
Tcpu=zeros(n,1);
Tgpu=zeros(n,3);
fprintf('----------------------------------------\n')
fprintf('Bench of generalized Newton fractals\n')
fprintf('  base : Matlab not vectorized version\n')
fprintf('  vec  : Matlab vectorized version\n')
fprintf('  gpu  : GPU version\n')
fprintf('----------------------------------------\n')
fprintf('  f=%s, f''=%s  (matlab datas)\n', ...
        Thrust.tools.strfunhandle(f),Thrust.tools.strfunhandle(df));
fprintf('  tol=%g, itermax=%g, box=[%g,%g,%g,%g]\n',tol,itermax,box);
fprintf('----------------------------------------\n')
fprintf('1) Starting benchmark : \n')
for i=1:n
  fprintf(' (%d/%d) on %d-by-%d grid\n',i,n,LN(i,:));
  z0=complexGrid(box,LN(i,:));
  [~,~,Tv]=GeneralizedNewtonFractalVec(f,df,z0,'tol',tol, 'itermax',itermax,'a',a);
  Tcpu(i)=Tv;
  fprintf(' vec: %.3f(s),',Tv);
  [~,~,T]=GeneralizedNewtonFractalGPUsimple(z0,'tol',tol, 'itermax',itermax,'a',a);
  Tgpu(i,:)=T;
  fprintf(' gpu: %.3f(s)\n',sum(T));
  fprintf(' gpu details: CPU->GPU %.3f(s), GPU computation: %.3f(s), GPU->CPU: %.3f(s)\n',T);
  fprintf(' -> speedup: x%3.2f\n',Tv/sum(T));
end
