import Thrust.samples.NewtonFractal.*
P=Thrust.tools.getCurrentFunctionPath();

DevProp=cuda.GetDeviceProperties();
fprintf('Generalized Newton Fractals on GPU %s\n',DevProp.Name);

fprintf('Compilation of the CUDA kernel ...\n');
tstart=tic();
%file=create_funcs('z*z*z-1.0','3.*z*z');
cudamexfile=[P,'mex_GeneralizedNewtonFractal_fullV2.cu'];
dep=[P,'funcs.h'];
%Thrust.compile.mex(cudamexfile,'samepath',true,'verbose',0,'dep',dep)
Tcomp=toc(tstart);
fprintf(' -> Compilation in %.3f(s)\n',Tcomp);
tol=1e-12;
itermax=100;
N=[2500,2500];
a=1;
%a=complex(1,1);

x=linspace(-2,2,N(1));
y=linspace(-2,2,N(2));
[xGrid,yGrid] = meshgrid( x, y );
z0 = xGrid + 1i*yGrid;


fprintf(' image size %dx%d\n',N)
fprintf('1) Transfert/create datas on GPU\n')
cuda.DeviceSynchronize();
tstart=tic();
gz0=mThrust(z0(:),'device'); % Only vectors are supported on mThrust
gz=mThrust(numel(z0),'device','double',true);
gcount=mThrust(numel(z0),'device','unsigned',false);
cuda.DeviceSynchronize();
T(1)=toc(tstart);
fprintf('2) Computation on GPU\n');
tstart=tic();
%mex_GeneralizedNewtonFractal_fullV2(complex(a),gz0,uint32(itermax),tol,gz,gcount);
%mex_GeneralizedNewtonFractal_fullV3(complex(a),gz0,uint32(itermax),tol,gz,gcount);
%mex_GeneralizedNewtonFractal(complex(a),gz0,uint32(itermax),tol,gz,gcount);
mex_GeneralizedNewtonFractal_full(complex(a),gz0,uint32(itermax),tol,gz,gcount);
cuda.DeviceSynchronize();
T(2)=toc(tstart);
fprintf('3) Transfert datas on Matlab\n')
tstart = tic();
z=reshape(gather(gz),size(z0));
count=reshape(gather(gcount),size(z0));
cuda.DeviceSynchronize();
T(3) = toc( tstart ); % transfert time to CPU
fprintf('   to GPU : %.3f(s), computation: %.3f(s), to CPU: %.3f(s)\n',T)
fprintf('  -> total: %.3f(s)\n',sum(T));


figure(1)
image(x,y,count,'CDataMapping','scaled')                     
colorbar
colormap('jet')
title('Number of iterations')

% Basins of attraction of the roots
[AlphaC,Num]=findZeros(z,count,itermax,tol);

figure(2)
image(x,y,Num,'CDataMapping','scaled')
colormap('jet')
title('Basins of attraction of the roots')
