import Thrust.samples.NewtonFractal.*
P=Thrust.tools.getCurrentFunctionPath();
file=create_funcs('z*z*z*z*z-1.0','5.*z*z*z*z');

cudamexfile=[P,'mex_GeneralizedNewtonFractal.cu'];
Thrust.compile.mex(cudamexfile,'samepath',true,'verbose',1,'dep',file)


tol=1e-12;
itermax=500;
N=2500;
p=5;
f=@(z) (z*z*z*z*z-1);
fp=@(z) p*z*z*z*z;
a=1;

x=linspace(-1.5,1.5,N);
y=linspace(-1.5,1.5,N);
[xGrid,yGrid] = meshgrid( x, y );
z0 = xGrid + 1i*yGrid;
fprintf('Comptation on CPU \n');
tstart=tic();
[z,S]=GeneralizedNewtonFractal(f,fp,z0,'tol',tol,'itermax',itermax,'verbose',0,'a',1);
T(1)=toc(tstart);
fprintf('Newton CPU : %.3f(s)\n',T(1))
[Alpha,num]=findZeros(z,S.count,S.itermax,S.tol);

DevProp=cuda.GetDeviceProperties();
fprintf('Comptation on GPU %s\n',DevProp.Name);
cuda.DeviceSynchronize();
tstart=tic();
gz0=mThrust(z0(:),'device');
gz=mThrust(numel(z0),'device','double',true);
gcount=mThrust(numel(z0),'device','unsigned',false);
cuda.DeviceSynchronize();
T(2)=toc(tstart);
tstart=tic();
mex_GeneralizedNewtonFractal(complex(a),gz0,uint32(itermax),tol,gz,gcount);
cuda.DeviceSynchronize();
T(3)=toc(tstart);
tstart = tic();
Z=reshape(gather(gz),size(z));
Count=reshape(gather(gcount),size(z));
cuda.DeviceSynchronize();
T(4) = toc( tstart ); % transfert time to CPU
fprintf('   to GPU : %.3f(s), computation: %.3f(s), to CPU: %.3f(s)\n',T(2:4))
fprintf('  -> total: %.3f(s)\n',sum(T(2:4)));

[dAlpha,dnum]=findZeros(Z,Count,itermax,tol);

figure(1)
image(x,y,S.count,'CDataMapping','scaled')
colormap('jet'); % colormap('hot');
colorbar
axis equal
axis off
title('CPU computation')

figure(2)
image(x,y,num,'CDataMapping','scaled')
colormap('jet'); % colormap('hot');
axis equal
axis off
title('CPU computation')

figure(3)
image(x,y,Count,'CDataMapping','scaled')
colormap('jet'); % colormap('hot');
colorbar
axis equal
axis off
title('GPU computation')

figure(4)
image(x,y,dnum,'CDataMapping','scaled')
colormap('jet'); % colormap('hot');
axis equal
axis off
title('GPU computation')