function varargout=eq(varargin)
  p = inputParser; 
  p.addParamValue('verbose', 0, @(x) isscalar(x) & isnumeric(x));
  p.parse(varargin{:});
  verbose=p.Results.verbose;
  name=mfilename();
  if verbose>1, fprintf('TEST : "%s" function\n',name); end
  res=test1(verbose);
  res=res+test2(verbose);
%    res=res+test3(verbose);
%    res=res+test4(verbose);
%    res=res+test5(verbose);
%    res=res+test6(verbose);
%    res=res+test7(verbose);
%    res=res+test8(verbose);
%    res=res+test9(verbose);
  Thrust.valid.tools.printFinalRes(name,res,verbose)
  if nargout==1, varargout{1}=res;end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test1(verbose)
  if verbose>1, fprintf(' test 1 : mThrust == mThrust\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types()%'type','numerical')
    for container=C
      fail=fail+test1_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test1_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 1 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test1_simple(container,type,iscomplex,verbose) 
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  if iscomplex
    x=complex(Conv(randi(100,dim,1)),Conv(randi(100,dim,1)));
    y=complex(Conv(randi(100,dim,1)),Conv(randi(100,dim,1)));
  else
    if strcmp(type,'logical')
      x=Conv(randi(100,dim,1)>=50);
      y=Conv(randi(100,dim,1)>=50);
    else
      x=Conv(randi(100,dim,1));y=Conv(randi(100,dim,1));
    end
  end
  gx=mThrust(x,container);
  gy=mThrust(y,container);
  startline='  ';
  cmd=sprintf('gz=(gx==gy);');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gy is %s\n',startline,prop(gy));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gz);
  if sum((x==y)==z)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gz is %s',startline,prop(gz));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test2(verbose)
  if verbose>1, fprintf(' test 2 : mThrust == cst\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types()%'type','numerical')
    for container=C
      fail=fail+test2_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test2_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 2 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test2_simple(container,type,iscomplex,verbose) 
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  N=2;
  dim=randi(10^5);
%  dim=5;
  if iscomplex
    x=complex(Conv(randi(N,dim,1)),Conv(randi(N,dim,1)));
    y=complex(Conv(randi(N,1,1)),Conv(randi(N,1,1)));
  else
    if strcmp(type,'logical')
      x=Conv(randi(N,dim,1)>=50);
      y=Conv(randi(N,1,1)>=50);
    else
      x=Conv(randi(N,dim,1));y=Conv(randi(N,1,1));
    end
  end
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('gz=(gx==y);');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% y is %s\n',startline,arrayprop(y));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gz);
%    gz
%    x
%    gx
%    y
%    x==y
%  
%    sum((x==y)==z)
  if sum((x==y)==z)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gz is %s',startline,prop(gz));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

