function varargout=math(varargin)
  p = inputParser; 
  p.addParamValue('fun','erf' , @ischar);
  p.addParamValue('verbose', 0, @(x) isscalar(x) & isnumeric(x));
  p.parse(varargin{:});
  verbose=p.Results.verbose;fun=p.Results.fun;
  name=mfilename();
  if verbose>1, fprintf('TEST : "%s" function\n',fun); end
  res=test1(fun,verbose);
  Thrust.valid.tools.printFinalRes(fun,res,verbose)
  if nargout==1, varargout{1}=res;end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test1(cfun,verbose)
  if verbose>1, fprintf(' test 1 : %s(mThrust)\n',cfun); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','real')
    for container=C
      fail=fail+test1_simple(cfun,container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1,
    S=sprintf(' test 1 : [%d/%d]',count-fail,count); 
    line=repmat('.',1,70-length(S));
    if (res(1)==0)
      fprintf('%s%sOK\n',S,line);
    else
      fprintf('%s%sFailed\n',S,line);
    end
  end
end
  
function failure=test1_simple(cfun,container,type,iscomplex,verbose) 
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  x=Thrust.valid.generate(dim,type,iscomplex);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('gy=%s(gx);',cfun);
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  y=gather(gy);
  eval(sprintf('res=%s(x);',cfun));
  Error=max(abs(double(res)-double(y))/(1+max(abs(double(y)))));
  Tol=50*Thrust.epstype(type);
  if ( Error > Tol )
    fprintf('  test1 with %s function\n',cfun)
    fprintf('    container=%s, type=%s, complex=%s, dim=%d\n',container,type,Thrust.tools.logical2str(iscomplex),dim)
    fprintf('    -> Max relative error : %e\n',Error)
    fprintf('    -> Selected tolerance : %e\n',Tol);
    idx=find(abs(res-y) == max(abs(res-y)) );
    failure=1;
  end
  %if sum(abs(res-y)<10*Thrust.toCtype(1,type))<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gy is %s',startline,prop(gy));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end
