function varargout=mtimes(varargin)
  p = inputParser; 
  p.addParamValue('verbose', 0, @(x) isscalar(x) & isnumeric(x));
  p.parse(varargin{:});
  verbose=p.Results.verbose;
  name=mfilename();
  if verbose>0, fprintf('TEST : "%s" function\n',name); end
  res=test1(verbose);
%    res=res+test2(verbose);
%    res=res+test3(verbose);
%    res=res+test4(verbose);
%    res=res+test5(verbose);
%    res=res+test6(verbose);
%    res=res+test7(verbose);
%    res=res+test8(verbose);
%    res=res+test9(verbose);
  if verbose>0, fprintf('TEST : "%s" function [%d/%d] OK\n',name,res(2)-res(1),res(2)); end
  if nargout==1, varargout{1}=res;end
end

function res=test1(verbose)
  if verbose>1, fprintf(' test 1 : mThrust .* mThrust\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test1_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','real')
    for container=C
      fail=fail+test1_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 1 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test1_simple(container,type,iscomplex,verbose) 
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  x=valid.generate(dim,type,iscomplex);
  y=valid.generate(dim,type,iscomplex);
  gx=mThrust(x,container);
  gy=mThrust(y,container);
  startline='  ';
  cmd=sprintf('gz=gx.*gy;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gy is %s\n',startline,prop(gy));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gz);
  
  if sum(abs(double(x).*double(y)-double(z))<1e-6)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gz is %s',startline,prop(gz));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test2(verbose)
  if verbose>1, fprintf(' test 2 : mThrust + constant[real]\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test2_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','real')
    for container=C
      fail=fail+test2_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 2 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test2_simple(container,type,iscomplex,verbose) 
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  if iscomplex
    x=complex(Conv(randi(100,dim,1)),Conv(randi(100,dim,1)));
    %a=complex(Conv(randi(100)),Conv(randi(100)));
  else
    x=Conv(randi(100,dim,1));
    %a=randi(100);
  end
  a=randi(100);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('gz=gx+a;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% a is %s\n',startline,arrayprop(a));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gz);
  if sum(double(x)+double(a)==double(z))<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gz is %s',startline,prop(gz));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test3(verbose)
  if verbose>1, fprintf(' test 3 : constant[real] + mThrust\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test2_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','real')
    for container=C
      fail=fail+test3_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 3 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test3_simple(container,type,iscomplex,verbose) 
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  if iscomplex
    x=complex(Conv(randi(100,dim,1)),Conv(randi(100,dim,1)));
    %a=complex(Conv(randi(100)),Conv(randi(100)));
  else
    x=Conv(randi(100,dim,1));
    %a=randi(100);
  end
  a=randi(100);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('gz=a+gx;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% a is %s\n',startline,arrayprop(a));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gz);
  if sum(double(x)+double(a)==double(z))<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gz is %s',startline,prop(gz));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test4(verbose)
  if verbose>1, fprintf(' test 4 : mThrust + constant[complex]\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test4_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','real')
    for container=C
      fail=fail+test4_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 4 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test4_simple(container,type,iscomplex,verbose) 
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  if iscomplex
    x=complex(Conv(randi(100,dim,1)),Conv(randi(100,dim,1)));
    %a=complex(Conv(randi(100)),Conv(randi(100)));
  else
    x=Conv(randi(100,dim,1));
    %a=randi(100);
  end
  a=complex(Conv(randi(100)),Conv(randi(100)));
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('gz=gx+a;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% a is %s\n',startline,arrayprop(a));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gz);
  if sum(abs(double(x)+double(a)-double(z))==0)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gz is %s',startline,prop(gz));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test5(verbose)
  if verbose>1, fprintf(' test 5 : constant[complex] + mThrust\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test5_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','real')
    for container=C
      fail=fail+test5_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 5 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test5_simple(container,type,iscomplex,verbose) 
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  if iscomplex
    x=complex(Conv(randi(100,dim,1)),Conv(randi(100,dim,1)));
    %a=complex(Conv(randi(100)),Conv(randi(100)));
  else
    x=Conv(randi(100,dim,1));
    %a=randi(100);
  end
  a=complex(Conv(randi(100)),Conv(randi(100)));
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('gz=a+gx;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% a is %s\n',startline,arrayprop(a));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gz);
  if sum(abs(double(x)+double(a)-double(z))==0)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gz is %s',startline,prop(gz));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test6(verbose)
  if verbose>1, fprintf(' test 6 : mThrust + mThrust<logical>\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test6_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','real')
    for container=C
      fail=fail+test6_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 6 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test6_simple(container,type,iscomplex,verbose) 
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  
  if iscomplex
    x=complex(Conv(randi(100,dim,1)),Conv(randi(100,dim,1)));
    %a=complex(Conv(randi(100)),Conv(randi(100)));
  else
    x=Conv(randi(100,dim,1));
    %a=randi(100);
  end
  bool=( randi(2,dim,1)==randi(2,dim,1) );
  gx=mThrust(x,container);
  gbool=mThrust(bool,container);
  startline='  ';
  cmd=sprintf('gz=gx+gbool;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gbool is %s\n',startline,prop(gbool));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gz);
  if sum(abs(double(x)+double(bool)-double(z))==0)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gz is %s',startline,prop(gz));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test7(verbose)
  if verbose>1, fprintf(' test 7 : mThrust<logical> + mThrust\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test7_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','real')
    for container=C
      fail=fail+test7_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 7 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test7_simple(container,type,iscomplex,verbose) 
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  
  if iscomplex
    x=complex(Conv(randi(100,dim,1)),Conv(randi(100,dim,1)));
    %a=complex(Conv(randi(100)),Conv(randi(100)));
  else
    x=Conv(randi(100,dim,1));
    %a=randi(100);
  end
  bool=( randi(2,dim,1)==randi(2,dim,1) );
  gx=mThrust(x,container);
  gbool=mThrust(bool,container);
  startline='  ';
  cmd=sprintf('gz=gbool+gx;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gbool is %s\n',startline,prop(gbool));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gz);
  if sum(abs(double(x)+double(bool)-double(z))==0)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gz is %s',startline,prop(gz));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test8(verbose)
  if verbose>1, fprintf(' test 8 : mThrust<complex> + mThrust<real>\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test8_simple(container{1},type{1},verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 8 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test8_simple(container,type,verbose) 
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  x=complex(Conv(randi(100,dim,1)),Conv(randi(100,dim,1)));
  y=Conv(randi(100,dim,1));
  gx=mThrust(x,container);
  gy=mThrust(y,container);
  startline='  ';
  cmd=sprintf('gz=gx+gy;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gy is %s\n',startline,prop(gy));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gz);
  if sum(abs(double(x)+double(y)-double(z))==0)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gz is %s',startline,prop(gz));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test9(verbose)
  if verbose>1, fprintf(' test 9 :  mThrust<real> + mThrust<complex> \n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test9_simple(container{1},type{1},verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 9 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test9_simple(container,type,verbose) 
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  x=complex(Conv(randi(100,dim,1)),Conv(randi(100,dim,1)));
  y=Conv(randi(100,dim,1));
  gx=mThrust(x,container);
  gy=mThrust(y,container);
  startline='  ';
  cmd=sprintf('gz=gy+gx;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gy is %s\n',startline,prop(gy));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gz);
  if sum(abs(double(x)+double(y)-double(z))==0)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gz is %s',startline,prop(gz));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end
