function varargout=sort(varargin)
  p = inputParser; 
  p.addParamValue('verbose', 0, @(x) isscalar(x) & isnumeric(x));
  p.parse(varargin{:});
  verbose=p.Results.verbose;
  name=mfilename();
  if verbose>1, fprintf('TEST : "%s" function\n',name); end
  res=test1(verbose);
  res=res+test2(verbose);
  res=res+test3(verbose);
  res=res+test4(verbose);
  res=res+test5(verbose);
  Thrust.valid.tools.printFinalRes(name,res,verbose)
  if nargout==1, varargout{1}=res;end
end

function res=test1(verbose)
  if verbose>1, fprintf(' test 1 : sort( <mThrust> )\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test1_simple(container{1},type{1},verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 1 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test1_simple(container,type,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^3);
  %dim=5;
  x=Thrust.valid.generate(dim,type,false);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('sort(gx);');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  sy=gather(gx);y=sort(x);
  Error=max(abs(double(sy)-double(y))/(1+max(abs(double(y)))));
  Tol=50*Thrust.epstype(type);
  if ( Error > Tol )
    failure=1;
  end
  if (verbose>2)
    S=sprintf('%s %% s is %s',startline,arrayprop(sy));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

% TEST 2
function res=test2(verbose)
  if verbose>1, fprintf(' test 2 : <mThrust> = sort( <mThrust> )\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test2_simple(container{1},type{1},verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 2 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test2_simple(container,type,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^3);
  %dim=5;
  x=Thrust.valid.generate(dim,type,false);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('gy=sort(gx);');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  sy=gather(gy);y=sort(x);
  Error=max(abs(double(sy)-double(y))/(1+max(abs(double(y)))));
  Tol=50*Thrust.epstype(type);
  if ( Error > Tol )
    failure=1;
  end
  if (verbose>2)
    S=sprintf('%s %% sy is %s',startline,arrayprop(sy));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

% TEST 3
function res=test3(verbose)
  if verbose>1, fprintf(' test 3 : [<mThrust>,<mThrust>] = sort( <mThrust> )\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test3_simple(container{1},type{1},verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 3 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test3_simple(container,type,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^3);
  %dim=5;
  x=Thrust.valid.generate(dim,type,false);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('[gy,gI]=sort(gx);');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  sy=gather(gy);sI=gather(gI)+1;
  [y,I]=sort(x);
  Error=max(abs(double(sy)-double(y))/(1+max(abs(double(y)))));
  Error=Error+max(abs(double(sI)-double(I)));
  Tol=50*Thrust.epstype(type);
  if ( Error > Tol )
    failure=1;
  end
  if (verbose>2)
    S=sprintf('%s %% sy is %s',startline,arrayprop(sy));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

% TEST 4
function res=test4(verbose)
  if verbose>1, fprintf(' test 4 : <mThrust> = sort( <mThrust> ,''mode'',''descend'')\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test4_simple(container{1},type{1},verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 4 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test4_simple(container,type,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^3);
  %dim=5;
  x=Thrust.valid.generate(dim,type,false);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('gy=sort(gx,''mode'',''descend'');');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  sy=gather(gy);y=sort(x,'descend');
  Error=max(abs(double(sy)-double(y))/(1+max(abs(double(y)))));
  Tol=50*Thrust.epstype(type);
  if ( Error > Tol )
    failure=1;
  end
  if (verbose>2)
    S=sprintf('%s %% sy is %s',startline,arrayprop(sy));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

% TEST 5
function res=test5(verbose)
  if verbose>1, fprintf(' test 5 : [<mThrust>,<mThrust>] = sort( <mThrust>,''mode'',''descend'' )\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test5_simple(container{1},type{1},verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 5 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test5_simple(container,type,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^3);
  %dim=5;
  x=Thrust.valid.generate(dim,type,false);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('[gy,gI]=sort(gx,''mode'',''descend'');');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  sy=gather(gy);sI=gather(gI)+1;
  [y,I]=sort(x,'descend');
  Error=max(abs(double(sy)-double(y))/(1+max(abs(double(y)))));
  Error=Error+max(abs(double(sI)-double(I)));
  Tol=50*Thrust.epstype(type);
  if ( Error > Tol )
    failure=1;
  end
  if (verbose>2)
    S=sprintf('%s %% sy is %s',startline,arrayprop(sy));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

