function varargout=sort_by_key(varargin)
  p = inputParser; 
  p.addParamValue('verbose', 0, @(x) isscalar(x) & isnumeric(x));
  p.parse(varargin{:});
  verbose=p.Results.verbose;
  name=mfilename();
  if verbose>1, fprintf('TEST : "%s" function\n',name); end
  res=test1(verbose);
  res=res+test2(verbose);
  res=res+test3(verbose);
  res=res+test4(verbose);
  Thrust.valid.tools.printFinalRes(name,res,verbose)
  if nargout==1, varargout{1}=res;end
end

%%%%% TEST 1 %%%%%
function res=test1(verbose)
  if verbose>1, fprintf(' test 1 : [<mThrust>,<mThrust>] = sort_by_key( <mThrust>,<mThrust> )\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for container=C
    for typeK=Thrust.types('type','numerical')
      for typeV=Thrust.types('type','numerical')
        fail=fail+test1_simple(container{1},typeK{1},typeV{1},false,verbose);
        count=count+1;
      end
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 1 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test1_simple(container,typeK,typeV,iscomplex,verbose)  
  ConvK=@(x) Thrust.toCtype(x,typeK);
  failure=0;
  dim=randi(10^3);
  %dim=5;
  x=Thrust.valid.generate(dim,typeK,iscomplex);
  gx=mThrust(x,container);
  y=Thrust.valid.generate(dim,typeV,iscomplex);
  gy=mThrust(y,container);
  startline='  ';
  cmd=sprintf('[sgx,sgy]=sort_by_key(gx,gy);');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gy is %s\n',startline,prop(gy));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  
  sy=gather(sgy);sx=gather(sgx);
  [shx,I]=sort(x);
  shy=y(I);
  Errorx=max(abs(double(sx)-double(shx))/(1+max(abs(double(shx)))));
  Tolx=50*Thrust.epstype(typeK);
  Errory=max(abs(double(sy)-double(shy))/(1+max(abs(double(shy)))));
  Toly=50*Thrust.epstype(typeV);
  if ( Errorx > Tolx & Errory > Toly )
    failure=1;
  end
  if (verbose>2)
    S=sprintf('%s %% sx is %s',startline,arrayprop(sx));
    fprintf('%s\n',S);
    S=sprintf('%s %% sy is %s',startline,arrayprop(sy));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%% TEST 2 %%%%%
function res=test2(verbose)
  if verbose>1, fprintf(' test 1 : sort_by_key( <mThrust>,<mThrust>,''inline'',true )\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for container=C
    for typeK=Thrust.types('type','numerical')
      for typeV=Thrust.types('type','numerical')
        fail=fail+test2_simple(container{1},typeK{1},typeV{1},false,verbose);
        count=count+1;
      end
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 2 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test2_simple(container,typeK,typeV,iscomplex,verbose)  
  ConvK=@(x) Thrust.toCtype(x,typeK);
  failure=0;
  dim=randi(10^3);
  %dim=5;
  x=Thrust.valid.generate(dim,typeK,iscomplex);
  gx=mThrust(x,container);
  y=Thrust.valid.generate(dim,typeV,iscomplex);
  gy=mThrust(y,container);
  startline='  ';
  cmd=sprintf('sort_by_key(gx,gy,''inline'',true);');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gy is %s\n',startline,prop(gy));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  
  sy=gather(gy);sx=gather(gx);
  [shx,I]=sort(x);
  shy=y(I);
  Errorx=max(abs(double(sx)-double(shx))/(1+max(abs(double(shx)))));
  Tolx=50*Thrust.epstype(typeK);
  Errory=max(abs(double(sy)-double(shy))/(1+max(abs(double(shy)))));
  Toly=50*Thrust.epstype(typeV);
  if ( Errorx > Tolx & Errory > Toly )
    failure=1;
  end
  if (verbose>2)
    S=sprintf('%s %% sx is %s',startline,arrayprop(sx));
    fprintf('%s\n',S);
    S=sprintf('%s %% sy is %s',startline,arrayprop(sy));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%% TEST 3 %%%%%
function res=test3(verbose)
  if verbose>1, fprintf(' test 3 : [<mThrust>,<mThrust>] = sort_by_key( <mThrust>,<mThrust>,''mode'',''descend'' )\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for container=C
    for typeK=Thrust.types('type','numerical')
      for typeV=Thrust.types('type','numerical')
        fail=fail+test3_simple(container{1},typeK{1},typeV{1},false,verbose);
        count=count+1;
      end
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 3 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test3_simple(container,typeK,typeV,iscomplex,verbose)  
  ConvK=@(x) Thrust.toCtype(x,typeK);
  failure=0;
  dim=randi(10^3);
  %dim=5;
  x=Thrust.valid.generate(dim,typeK,iscomplex);
  gx=mThrust(x,container);
  y=Thrust.valid.generate(dim,typeV,iscomplex);
  gy=mThrust(y,container);
  startline='  ';
  cmd=sprintf('[sgx,sgy]=sort_by_key(gx,gy,''mode'',''descend'');');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gy is %s\n',startline,prop(gy));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  
  sy=gather(sgy);sx=gather(sgx);
  [shx,I]=sort(x,'descend');
  shy=y(I);
  Errorx=max(abs(double(sx)-double(shx))/(1+max(abs(double(shx)))));
  Tolx=50*Thrust.epstype(typeK);
  Errory=max(abs(double(sy)-double(shy))/(1+max(abs(double(shy)))));
  Toly=50*Thrust.epstype(typeV);
  if ( Errorx > Tolx & Errory > Toly )
    failure=1;
  end
  if (verbose>2)
    S=sprintf('%s %% sx is %s',startline,arrayprop(sx));
    fprintf('%s\n',S);
    S=sprintf('%s %% sy is %s',startline,arrayprop(sy));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%% TEST 4 %%%%%
function res=test4(verbose)
  if verbose>1, fprintf(' test 4 : sort_by_key( <mThrust>,<mThrust>,''mode'',''descend'',''inline'',true )\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for container=C
    for typeK=Thrust.types('type','numerical')
      for typeV=Thrust.types('type','numerical')
        fail=fail+test4_simple(container{1},typeK{1},typeV{1},false,verbose);
        count=count+1;
      end
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 4 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test4_simple(container,typeK,typeV,iscomplex,verbose)  
  ConvK=@(x) Thrust.toCtype(x,typeK);
  failure=0;
  dim=randi(10^3);
  %dim=5;
  x=Thrust.valid.generate(dim,typeK,iscomplex);
  gx=mThrust(x,container);
  y=Thrust.valid.generate(dim,typeV,iscomplex);
  gy=mThrust(y,container);
  startline='  ';
  cmd=sprintf('sort_by_key(gx,gy,''mode'',''descend'',''inline'',true);');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gy is %s\n',startline,prop(gy));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  
  sy=gather(gy);sx=gather(gx);
  [shx,I]=sort(x,'descend');
  shy=y(I);
  Errorx=max(abs(double(sx)-double(shx))/(1+max(abs(double(shx)))));
  Tolx=50*Thrust.epstype(typeK);
  Errory=max(abs(double(sy)-double(shy))/(1+max(abs(double(shy)))));
  Toly=50*Thrust.epstype(typeV);
  if ( Errorx > Tolx & Errory > Toly )
    failure=1;
  end
  if (verbose>2)
    S=sprintf('%s %% sx is %s',startline,arrayprop(sx));
    fprintf('%s\n',S);
    S=sprintf('%s %% sy is %s',startline,arrayprop(sy));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

