function varargout=sum(varargin)
  p = inputParser; 
  p.addParamValue('verbose', 0, @(x) isscalar(x) & isnumeric(x));
  p.parse(varargin{:});
  verbose=p.Results.verbose;
  name=mfilename();
  if verbose>1, fprintf('TEST : "%s" function\n',name); end
  res=test1(verbose);
  Thrust.valid.tools.printFinalRes(name,res,verbose)
  if nargout==1, varargout{1}=res;end
end

function res=test1(verbose)
  if verbose>1, fprintf(' test 1 : sum( <mThrust> )\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types()
    for container=C
      fail=fail+test1_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test1_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 1 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test1_simple(container,type,iscomplex,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^3);
  %dim=5;
  x=Thrust.valid.generate(dim,type,iscomplex);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('s=sum(gx);');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  if ( abs(double(s)-double(sum(x)))/abs(double(s)) > 100*Thrust.epstype(type) )
    failure=1;
  end
  if (verbose>2)
    S=sprintf('%s %% s is %s',startline,arrayprop(s));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

