function obj=linspace(a,b,n,varargin)
%  THRUST.LINSPACE Linspace mThrust vector
%      THRUST.LINSPACE(A, B, N) generates N points between A and B.
%       
%      A and B must be a single or double scalar. Both real and complex types are supported.
%      
%      Examples:
%       
%          N  = 1000;
%          a = 0;
%          b = 10;
%          X = Thrust.linspace(a, b, N); 
%      
%      See also LINSPACE, MTHRUST.
  p = inputParser; 
  p.addParamValue('container', 'device', @(x) ismember(x,Thrust.getContainers()));
  p.addParamValue('type', 'double', @(x) ismember(x,Thrust.types('type','real')));
  p.parse(varargin{:});
  container=p.Results.container;
  type=p.Results.type;
  isReal=(isreal(a) & isreal(b));
  if ~isReal,a=complex(a);b=complex(b);end
  obj=mThrust(n,container,type,~isReal);
  mex_mThrust_linspace(obj,Thrust.toCtype(a,type),Thrust.toCtype((b-a)/(n-1),type));
end
