function obj=minus(obj1,obj2)
% +  mThrust vectors subtraction.
%    Z = X - Y or Z=minus(X,Y) substracts mThrust vectors Y from X.
%      - compatible mThrust vectors (same sizes, same types or one is logical)
%      - X a mThrust vector and Y a scalar 
%      - X a scalar and Y a mThrust vector
%    The scalar values are converted to the type of the mThrust vector.
%
%    Example
%       A=Thrust.rand(1000);
%       B=Thrust.rand(1000);
%       C1=A-2;
%       C2=2-A;
%       C3=A-B;
%       C4=A-(B>0.5)
%       C5=minus(A,B);
%
% Francois Cuvelier 2013-2016
  iscomplex=(~isreal(obj1) || ~isreal(obj2));
  if ( Thrust.ismThrust(obj1) && Thrust.ismThrust(obj2) ) % mThrust<type>  <-  mThrust<type> - mThrust<type>
    assert(obj1.dim==obj2.dim,'Dimension trouble');
    assert(strcmp(obj1.container,obj2.container),'container trouble');
    if (strcmp(obj2.type,'logical'))
      obj=mThrust(obj1.dim,obj1.container,obj1.type,iscomplex);
      mex_mThrust_minuslogical(obj1,obj2,obj);
    elseif (strcmp(obj1.type,'logical'))
      obj=mThrust(obj2.dim,obj2.container,obj2.type,iscomplex);
      mex_mThrust_logicalminus(obj2,obj1,obj);
    elseif (strcmp(obj1.type,obj2.type))
      obj=mThrust(obj1.dim,obj1.container,obj1.type,iscomplex);
      mex_mThrust_minus(obj1,obj2,obj);
    end
  elseif ( (Thrust.ismThrust(obj1)) && ( isnumeric(obj2) ) ) % mThrust<type>  <-  mThrust<type> - scalar<type>
    assert(length(obj2)==1,'dimension failed');
    A=Thrust.toCtype(obj2,obj1.type);
    obj=mThrust(obj1.dim,obj1.container,obj1.type,iscomplex);
    mex_mThrust_minuscst(obj1,A,obj);
  elseif ( (Thrust.ismThrust(obj2)) && ( isnumeric(obj1) ) ) % mThrust<type>  <-  scalar<type> - mThrust<type>
    assert(length(obj1)==1,'dimension failed');
    A=Thrust.toCtype(obj1,obj2.type);
    obj=mThrust(obj2.dim,obj2.container,obj2.type,iscomplex);
    mex_mThrust_cstminus(obj2,A,obj);
  else
    error('incompatible type')
  end
end