function obj=mtimes(obj1,obj2)
% *  mThrust vector multiplication by a scalar.
%    Z = X*Y or Z=mtimes(X,Y) denotes multiplication by a scalar.
%    One of the parameters must be a scalar the other a mThrust vector.
%    The scalar value is converted to the type of the mThrust vector.
%
%    Example
%       x = Thrust.sequence(10);
%       z1 = x*3;
%       z2 = 2*x;
%       z3 = (1+i)*x; % complex 
%
% Francois Cuvelier 2013-2016
  iscomplex=(~isreal(obj1) || ~isreal(obj2));
  if ( (Thrust.ismThrust(obj1)) && ( isnumeric(obj2) ) ) 
    assert(length(obj2)==1,'dimension failed');
    A=Thrust.toCtype(obj2,obj1.type);
    obj=mThrust(obj1.dim,obj1.container,obj1.type,iscomplex);
    mex_mThrust_mtimes(obj1,A,obj);
  elseif ( (Thrust.ismThrust(obj2)) && ( isnumeric(obj1) ) ) % gTArray<type>  <-  scalar<type> * gTArray<type>
    assert(length(obj1)==1,'dimension failed');
    A=Thrust.toCtype(obj1,obj2.type);
    obj=mThrust(obj2.dim,obj2.container,obj2.type,iscomplex);
    mex_mThrust_mtimes(obj2,A,obj);
  else
    error('incompatible type')
  end
end