function obj=or(obj1,obj2)
% |  Logical OR 
%    X | Y or OR(X,Y) performs the logical OR of mThrust logical vector A and B and returns
%    an mThrust logical vector. 
%    The output i-th element is set to logical 1 (true) if A(i)==1 or B(i)==1 otherwise 
%    is set to logical 1 (false).
%
%    Example
%       x = Thrust.rand(100);
%       z1 = (x<0.2) | (x>0.6);
%       z2 = or(x<0.2,x>0.6);
%
% Francois Cuvelier 2013-2016
  if ( Thrust.ismThrust(obj1) & Thrust.ismThrust(obj2) )
    assert(strcmp(obj1.type,'logical') & strcmp(obj2.type,'logical') & strcmp(obj1.container,obj2.container) & obj1.dim==obj2.dim, ...
           'mThrust:and incompatible objects')
    obj=mThrust(obj1.dim,obj1.container,'logical');
    mex_mThrust_or(obj1,obj2,obj);
    return
  end
  if ( Thrust.ismThrust(obj1) & islogical(obj2) )
    obj=mThrust(obj1.dim,obj1.container,'logical');
    mex_mThrust_or(obj1,obj2,obj);  
  end
  
end