function obj=power(obj1,obj2)
% .^ or POWER mThrust vector power.
%    Z = X.^Y or Z=power(X,Y) denotes element-by-element operation of X raised
%    to the Y power. X and Y can be 
%      - compatible mThrust vectors (same sizes and types)
%      - X a mThrust vector and Y a scalar 
%      - X a scalar and Y a mThrust vector
%    The scalar values are converted to the type of the mThrust vector.
%
%    Example
%       x = Thrust.sequence(10);
%       y = Thrust.linspace(10);
%       z0 = x.^y;
%       z1 = x.^3;
%       z2 = 2.^x;
%
% Francois Cuvelier 2013-2016
  iscomplex=(~isreal(obj1) || ~isreal(obj2));
  if ( Thrust.ismThrust(obj1) && Thrust.ismThrust(obj2) ) 
    assert(obj1.dim==obj2.dim,'Dimension trouble');
    assert(ismember(obj1.type,{'double','float'}),'Type must be float or double')
    assert(strcmp(obj1.type,obj2.type))
    obj=mThrust(obj1.dim,obj1.container,obj1.type,iscomplex);
    mex_mThrust_power(obj1,obj2,obj);
  elseif ( (Thrust.ismThrust(obj1)) && ( isnumeric(obj2) ) ) 
    assert(length(obj2)==1,'dimension failed');
    A=Thrust.toCtype(obj2,obj1.type);
    obj=mThrust(obj1.dim,obj1.container,obj1.type,iscomplex);
    mex_mThrust_powercst(obj1,A,obj);
  elseif ( (Thrust.ismThrust(obj2)) && ( isnumeric(obj1) ) ) 
    assert(length(obj1)==1,'dimension failed');
    A=Thrust.toCtype(obj1,obj2.type);
    obj=mThrust(obj2.dim,obj2.container,obj2.type,iscomplex);
    mex_mThrust_cstpower(A,obj2,obj);
  else
    error('incompatible type')
  end
end