function varargout=sort_by_key(keys,values,varargin)
% sort_by_keys  Performs a key-value sort on mThrust vectors.
%   [skeys,svalues]=sort_by_key(keys,values) 
%     Performs a key-value sort on the mThrust vectors keys and values 
%     by using thrust::sort_by_key function from the Thrust API. The output 
%     are the key-value sort of the input. The types of mThrust vectors 
%     keys and values can be differents.
%
%   sort_by_key(keys,values,'inline',true) Performs an inline key-value sort 
%   on the mThrust vectors keys and values (so they are modified). 
%
%   Example:
%     keys=Thrust.rand(10);
%     vals=Thrust.sequence(10,'type','int')
%     [sk,sv]=sort_by_key(keys,vals);
%
% see : http://thrust.github.io/doc/group__sorting.html
%
% Francois Cuvelier 2013-2016
  p = inputParser; 
  p.addParamValue('mode', 'ascend', @(x) ismember(x,{'ascend','descend'}));
  p.addParamValue('inline', false, @islogical); % experimental
  p.parse(varargin{:});
  mode=p.Results.mode;inline=p.Results.inline;
  if strcmp(mode,'ascend'), Flag=uint32(1);else Flag=uint32(0);end
  
  
  assert( (nargout==2) | (nargout==0) , 'mThrust:sort_by_key bad ouput')
  assert(Thrust.ismThrust(keys) & Thrust.ismThrust(values) ) 
  assert( ~keys.complex & ~values.complex)
  if nargout==2
    varargout{1}=mThrust(keys);
    varargout{2}=mThrust(values);
    mex_mThrust_sort_by_key(varargout{1},varargout{2},Flag);
  else
    if ~p.Results.inline
      warning('Must set ''inline'' option to true ...')
      return
    end
    mex_mThrust_sort_by_key(keys,values,Flag);
  end
end