function obj=times(obj1,obj2)
% .*  mThrust vector multiplication.
%    Z = X.*Y or Z=power(X,Y) denotes element-by-element multiplication. 
%    X and Y can be 
%      - compatible mThrust vectors (same sizes, same types or one is logical)
%      - X a mThrust vector and Y a scalar 
%      - X a scalar and Y a mThrust vector
%    The scalar values are converted to the type of the mThrust vector.
%
%    Example
%       x = Thrust.sequence(10);
%       y = Thrust.linspace(10);
%       z0 = x.*y;
%       z1 = x.*3;
%       z2 = 2.*x;
%       z3 = y.*(x>5);
%
% Francois Cuvelier 2013-2016
  iscomplex=(~isreal(obj1) || ~isreal(obj2));
  if ( Thrust.ismThrust(obj1) && Thrust.ismThrust(obj2) ) 
    assert(obj1.dim==obj2.dim,'Dimension trouble');
    %assert(strcmp(obj1.type,obj2.type),'Type trouble');
    assert(strcmp(obj1.container,obj2.container),'Container trouble');
    if strcmp(obj1.type,obj2.type)
      obj=mThrust(obj1.dim,obj1.container,obj1.type,iscomplex);
      mex_mThrust_times(obj1,obj2,obj);
    elseif strcmp(obj2.type,'logical')
      obj=mThrust(obj1.dim,obj1.container,obj1.type,iscomplex);
      mex_mThrust_timeslogical(obj1,obj2,obj);
    elseif strcmp(obj1.type,'logical')
      obj=mThrust(obj2.dim,obj2.container,obj2.type,iscomplex);
      mex_mThrust_timeslogical(obj2,obj1,obj);  
    else
      error('incompatible type')
    end
  elseif ( (Thrust.ismThrust(obj1)) && ( isnumeric(obj2) ) ) % mThrust<type>  <-  mThrust<type> .* scalar<type>
    assert(length(obj2)==1,'dimension failed');
    A=Thrust.toCtype(obj2,obj1.type);
    obj=mThrust(obj1.dim,obj1.container,obj1.type,iscomplex);
    mex_mThrust_mtimes(obj1,A,obj);
  elseif ( (Thrust.ismThrust(obj2)) && ( isnumeric(obj1) ) ) % mThrust<type>  <-  scalar<type> .* mThrust<type>
    assert(length(obj1)==1,'dimension failed');
    A=Thrust.toCtype(obj1,obj2.type);
    obj=mThrust(obj2.dim,obj2.container,obj2.type,iscomplex);
    mex_mThrust_mtimes(obj2,A,obj);
  else
    error('incompatible type')
  end
end