/* Generic for math function f:thrust::complex<T>->T 
 T could be float or double
 
 */

template <typename VectorIn,typename VectorOut> 
void mx_mThrust_funmath_gen(const mxArray *prhs[]){
  typedef typename VectorIn::value_type  Tin;
  typedef typename VectorOut::value_type Tout;
  VectorIn  &ga = get_object<VectorIn >(mxGetProperty(prhs[0],0,"handle"));
  VectorOut &gb = get_object<VectorOut>(mxGetProperty(prhs[1],0,"handle"));
  
  assert(gb.size()==ga.size());
  thrust::transform(ga.begin(),ga.end(),gb.begin(), cmath_functor<Tin,Tout>());
}


template  <typename T>
void mx_mThrust_mathfun_gen_type(const mxArray *prhs[]){
  char *container[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"container")),
    mxArrayToString(mxGetProperty(prhs[1],0,"container"))
  };
  assert( strcmp(container[0],container[1])==0 );
  bool iscomplex[2];
  for (int i=0;i<2;i++){
    logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[i],0,"complex"));
    iscomplex[i]=*iscomp;
  }
  assert( !iscomplex[1] );
  
  if (strcmp(container[0], "device")==0){
    typedef thrust::device_vector<thrust::complex<T> > VecCplx;
    typedef thrust::device_vector<T> VecReal;
    if (iscomplex[0])
      mx_mThrust_funmath_gen<VecCplx,VecReal>(prhs); 
    else
      mx_mThrust_funmath_gen<VecReal,VecReal>(prhs);
  }else if (strcmp(container[0], "host")==0){
    typedef thrust::host_vector<thrust::complex<T> > VecCplx;
    typedef thrust::host_vector<T> VecReal;
    if (iscomplex[0])
      mx_mThrust_funmath_gen<VecCplx,VecReal>(prhs); 
    else
      mx_mThrust_funmath_gen<VecReal,VecReal>(prhs);  
  }else{
    mxFree(container[0]);mxFree(container[1]);
    mexErrMsgIdAndTxt(MESS,"Unknow container."); 
  }
  mxFree(container[0]);mxFree(container[1]);
}

// prhs[0] : input mThrust object
// prhs[1] : output mThrust object
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 2) 
    mexErrMsgTxt("Bad input... To do");
  
  assert( mxIsClass(prhs[0],"mThrust") && mxIsClass(prhs[1],"mThrust") );
  
  char *type[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"type")),
    mxArrayToString(mxGetProperty(prhs[1],0,"type"))
  };
  assert( strcmp(type[0],type[1])==0 );
  
  if (strcmp(type[0], "double")==0)
    mx_mThrust_mathfun_gen_type<double>(prhs);
  else if (strcmp(type[0], "float")==0)
    mx_mThrust_mathfun_gen_type<float>(prhs);
  //else if (strcmp(type[0], "int")==0)
  //  mx_mThrust_mathfun_gen_type<int>(prhs);
  //else if (strcmp(type[0], "unsigned")==0)
  //  mx_mThrust_mathfun_gen_type<unsigned>(prhs);
  else{
    mxFree(type[0]);mxFree(type[1]);
    mexErrMsgIdAndTxt(MESS,"type not implemented.");
  }
  mxFree(type[0]);mxFree(type[1]);
  MEXPRINTF("ending");
}