/* use by mex_mThrust_cummax,  mex_mThrust_cummin
     (not complex numbers!)
 */

template <typename Vector> void mx_mThrust_cumfun_gen(const mxArray *prhs[]){
  typedef typename Vector::value_type T;
  Vector &Vi = get_object< Vector >(mxGetProperty(prhs[0],0,"handle")); // input
  Vector &Vo = get_object< Vector >(mxGetProperty(prhs[1],0,"handle")); // output
  
  assert(Vi.size()==Vo.size());
  thrust::inclusive_scan(Vi.begin(),Vi.end(),Vo.begin(),cum_functor<T>());
}


template  <typename T>
void mx_mThrust_cumfun_gen_type(const mxArray *prhs[]){
  char *container[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"container")),
    mxArrayToString(mxGetProperty(prhs[1],0,"container"))
  };
  assert( strcmp(container[0],container[1])==0 );
  bool iscomplex[2];
  for (int i=0;i<2;i++){
    logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[i],0,"complex"));
    iscomplex[i]=*iscomp;
  }
  assert( !iscomplex[0]  && !iscomplex[1] );
  
  if (strcmp(container[0], "device")==0){
    typedef thrust::device_vector<T> VecReal;
    mx_mThrust_cumfun_gen<VecReal>(prhs); 
  }else if (strcmp(container[0], "host")==0){
    typedef thrust::host_vector<T> VecReal;
    mx_mThrust_cumfun_gen<VecReal>(prhs);  
  }else
    mexWarnMsgIdAndTxt(MESS,"Unknow container."); 
  mxFree(container[0]);mxFree(container[1]);
}

// prhs[0] : handle to input mThrust object
// prhs[2] : handle to output mThrust object
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 2) 
    mexErrMsgTxt("Bad input... To do");
  
  assert( mxIsClass(prhs[0],"mThrust") && mxIsClass(prhs[1],"mThrust") );
  
  char *type[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"type")),
    mxArrayToString(mxGetProperty(prhs[1],0,"type"))
  };
  assert( strcmp(type[0],type[1])==0 );
  
  if (strcmp(type[0], "double")==0)
    mx_mThrust_cumfun_gen_type<double>(prhs);
  else if (strcmp(type[0], "float")==0)
    mx_mThrust_cumfun_gen_type<float>(prhs);
  else if (strcmp(type[0], "int")==0)
    mx_mThrust_cumfun_gen_type<int>(prhs);
  else if (strcmp(type[0], "unsigned")==0)
    mx_mThrust_cumfun_gen_type<unsigned>(prhs);
  else
    mexWarnMsgIdAndTxt(MESS,"type not implemented.");
  mxFree(type[0]);mxFree(type[1]);
  MEXPRINTF("ending");
}