#include <string>
//#include <sstream>
#include <cuda.h>
//#include <helper_cuda.h>
#include "textcolors.h"
#include "mex.h"
//#define VERBOSE
#include <stdio.h>
 
// From helper_cuda.h
// Beginning of GPU Architecture definitions
inline int _ConvertSMVer2Cores(int major, int minor)
{
  // Defines for GPU Architecture types (using the SM version to determine the # of cores per SM
  typedef struct
  {
    int SM; // 0xMm (hexidecimal notation), M = SM Major version, and m = SM minor version
    int Cores;
  } sSMtoCores;
  
  sSMtoCores nGpuArchCoresPerSM[] =
  {
    { 0x20, 32 }, // Fermi Generation (SM 2.0) GF100 class
    { 0x21, 48 }, // Fermi Generation (SM 2.1) GF10x class
    { 0x30, 192}, // Kepler Generation (SM 3.0) GK10x class
    { 0x32, 192}, // Kepler Generation (SM 3.2) GK10x class
    { 0x35, 192}, // Kepler Generation (SM 3.5) GK11x class
    { 0x37, 192}, // Kepler Generation (SM 3.7) GK21x class
    { 0x50, 128}, // Maxwell Generation (SM 5.0) GM10x class
    { 0x52, 128}, // Maxwell Generation (SM 5.2) GM20x class
    {   -1, -1 }
  };
  
  int index = 0;
  
  while (nGpuArchCoresPerSM[index].SM != -1)
  {
    if (nGpuArchCoresPerSM[index].SM == ((major << 4) + minor))
    {
      return nGpuArchCoresPerSM[index].Cores;
    }
    
    index++;
  }
  
  // If we don't find the values, we default use the previous one to run properly
  mexPrintf("MapSMtoCores for SM %d.%d is undefined.  Default to use %d Cores/SM\n", major, minor, nGpuArchCoresPerSM[index-1].Cores);
  return nGpuArchCoresPerSM[index-1].Cores;
}
// end of GPU Architecture definitions

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
  {
    if (nlhs >1 || nrhs > 1) 
      mexErrMsgTxt("Bad input... To do");
    
    if (nrhs==1)
      if( !mxIsInt32(prhs[0]) || mxGetNumberOfElements(prhs[0])!=1 ) {
        mexErrMsgIdAndTxt("mThrustToolbox:Init","Input value must be an int32.");
      }
  // -> Initialize output structure
  mwSize ndim = 2;
  const mwSize dims[2] = {1, 1};
  int number_of_fields=15;   
  const char *field_names[] = {"Name", "Index","ComputeCapability","RuntimeVersion","DriverVersion",
    "TotalMemory","AvailableMemory","MultiprocessorCount","CoresByMP","CudaCores","GPUMaxClockRateMhz",
    "MaxThreadsPerBlock","SharedMemPerBlock","MaxThreadBlockSize","MaxGridSize" };
  mxArray *struct_array_ptr;
  struct_array_ptr = mxCreateStructArray(ndim, dims, 
                                         number_of_fields, field_names);
  plhs[0]=struct_array_ptr;
  // <- Initialize output structure
  double *pn;
  pn= (double *) mxGetPr(prhs[0]);
  int n= (int) *pn; // Device Number
  int nn =(int) *( (double *) mxGetPr(prhs[0]));
  mexPrintf("n = %d - %d\n",n,nn);
  int devCount;
  cudaGetDeviceCount(&devCount);
  if (devCount == 0)
  {
    mexPrintf("There are no available device(s) that support CUDA\n");
    return;
  }
  
  mexPrintf("CUDA Device Query...\n");
  mexPrintf("There are %d CUDA devices.\n", devCount);
  
  if (n <= devCount){
    cudaSetDevice(n);
  // Get device properties
    cudaDeviceProp deviceProp;
    cudaGetDeviceProperties(&deviceProp, n);
    
    size_t free_byte, total_byte ;
    cudaMemGetInfo( &free_byte, &total_byte ) ;
    
    mexPrintf("\nSelect CUDA Device #%d : %s\n", n,deviceProp.name);
    mxSetField(struct_array_ptr, 0, "Name", mxCreateString(deviceProp.name));
    int driverVersion = 0, runtimeVersion = 0;
    cudaDriverGetVersion(&driverVersion);
    cudaRuntimeGetVersion(&runtimeVersion);
    mexPrintf("  CUDA Driver Version / Runtime Version          %d.%d / %d.%d\n", driverVersion/1000, (driverVersion%100)/10, runtimeVersion/1000, (runtimeVersion%100)/10);
    mexPrintf("  CUDA Capability Major/Minor version number:    %d.%d\n", deviceProp.major, deviceProp.minor);
    mxArray  *DriverVersion = mxCreateDoubleMatrix(1, 1, mxREAL);
    mxArray  *RuntimeVersion = mxCreateDoubleMatrix(1, 1, mxREAL);
    double    *pr;char cTemp[16];
    mxSetField(struct_array_ptr, 0, "Index", mxCreateDoubleScalar(n));
    mxSetField(struct_array_ptr, 0, "DriverVersion", mxCreateDoubleScalar(driverVersion/1000+(driverVersion%100)/100.));
    mxSetField(struct_array_ptr, 0, "RuntimeVersion", mxCreateDoubleScalar(runtimeVersion/1000+(runtimeVersion%100)/100.));
    mxSetField(struct_array_ptr, 0, "TotalMemory",mxCreateDoubleScalar((double)deviceProp.totalGlobalMem));
    mxSetField(struct_array_ptr, 0, "AvailableMemory",mxCreateDoubleScalar((double)(free_byte)));
    
    mxSetField(struct_array_ptr, 0, "MultiprocessorCount",mxCreateDoubleScalar((double)deviceProp.multiProcessorCount));
    mxSetField(struct_array_ptr, 0, "CoresByMP",mxCreateDoubleScalar((double)_ConvertSMVer2Cores(deviceProp.major, deviceProp.minor)));
    mxSetField(struct_array_ptr, 0, "CudaCores",mxCreateDoubleScalar((double)(_ConvertSMVer2Cores(deviceProp.major, deviceProp.minor)* deviceProp.multiProcessorCount)));
    mxSetField(struct_array_ptr, 0, "GPUMaxClockRateMhz",mxCreateDoubleScalar((double)(deviceProp.clockRate * 1e-3f)));
    
    mxSetField(struct_array_ptr, 0, "MaxThreadsPerBlock",mxCreateDoubleScalar((double)(deviceProp.maxThreadsPerBlock)));
    mxSetField(struct_array_ptr, 0, "SharedMemPerBlock",mxCreateDoubleScalar((double)(deviceProp.sharedMemPerBlock)));
    mxArray  *MaxThreadBlockSize = mxCreateDoubleMatrix(1, 3, mxREAL);
    pr = mxGetPr(MaxThreadBlockSize);
    for (int i=0;i<3;i++) pr[i]=deviceProp.maxThreadsDim[i];
    mxSetField(struct_array_ptr, 0, "MaxThreadBlockSize", MaxThreadBlockSize);
    mxSetField(struct_array_ptr, 0, "MaxGridSize", mxCreateDoubleMatrix(1, 3, mxREAL));
    pr=mxGetPr(mxGetField(struct_array_ptr, 0, "MaxGridSize"));
    for (int i=0;i<3;i++) pr[i]=deviceProp.maxGridSize[i];
    //mxSetField(struct_array_ptr, 0, "MaxThreadBlockSize",mxCreateDoubleScalar((double)(deviceProp.clockRate * 1e-3f)));
    //mxSetField(struct_array_ptr, 0, "MaxGridSize",mxCreateDoubleScalar((double)(deviceProp.clockRate * 1e-3f)));
//     mxArray  *ComputeCapability = mxCreateDoubleMatrix(1, 1, mxREAL);
//     pr = mxGetPr(ComputeCapability);
//     pr[0] = deviceProp.major+ (deviceProp.minor)/10.;
//    S1= std::to_string(deviceProp.major)+'.'+std::to_string(deviceProp.minor);
    //sscanf(tmp,"%d.%d",deviceProp.major,deviceProp.minor);
    #if defined(WIN32) || defined(_WIN32) || defined(WIN64) || defined(_WIN64)
    sprintf_s(cTemp, 10, "%d.%d", deviceProp.major, deviceProp.minor);
    #else
    sprintf(cTemp, "%d.%d", deviceProp.major, deviceProp.minor);
    #endif    
    mxSetField(struct_array_ptr, 0, "ComputeCapability", mxCreateString( (const char *)cTemp));
    char msg[256];
    sprintf(msg, "  Total amount of global memory:                 %.0f MBytes (%llu bytes)\n",
            (float)deviceProp.totalGlobalMem/1048576.0f, (unsigned long long) deviceProp.totalGlobalMem);
    mexPrintf("%s", msg);
    mexPrintf("  (%2d) Multiprocessors, (%3d) CUDA Cores/MP:     %d CUDA Cores\n",
           deviceProp.multiProcessorCount,
           _ConvertSMVer2Cores(deviceProp.major, deviceProp.minor),
           _ConvertSMVer2Cores(deviceProp.major, deviceProp.minor) * deviceProp.multiProcessorCount);
    mexPrintf("  GPU Max Clock rate:                            %.0f MHz (%0.2f GHz)\n", deviceProp.clockRate * 1e-3f, deviceProp.clockRate * 1e-6f);
    
    
    #if CUDART_VERSION >= 5000
    // This is supported in CUDA 5.0 (runtime API device properties)
    mexPrintf("  Memory Clock rate:                             %.0f Mhz\n", deviceProp.memoryClockRate * 1e-3f);
    mexPrintf("  Memory Bus Width:                              %d-bit\n",   deviceProp.memoryBusWidth);
    
    if (deviceProp.l2CacheSize)
    {
      mexPrintf("  L2 Cache Size:                                 %d bytes\n", deviceProp.l2CacheSize);
    }
    
    #else
    // This only available in CUDA 4.0-4.2 (but these were only exposed in the CUDA Driver API)
    int memoryClock;
    getCudaAttribute<int>(&memoryClock, CU_DEVICE_ATTRIBUTE_MEMORY_CLOCK_RATE, dev);
    mexPrintf("  Memory Clock rate:                             %.0f Mhz\n", memoryClock * 1e-3f);
    int memBusWidth;
    getCudaAttribute<int>(&memBusWidth, CU_DEVICE_ATTRIBUTE_GLOBAL_MEMORY_BUS_WIDTH, dev);
    mexPrintf("  Memory Bus Width:                              %d-bit\n", memBusWidth);
    int L2CacheSize;
    getCudaAttribute<int>(&L2CacheSize, CU_DEVICE_ATTRIBUTE_L2_CACHE_SIZE, dev);
    
    if (L2CacheSize)
    {
      mexPrintf("  L2 Cache Size:                                 %d bytes\n", L2CacheSize);
    }
    
    #endif
    mexPrintf("  Maximum Texture Dimension Size (x,y,z)         1D=(%d), 2D=(%d, %d), 3D=(%d, %d, %d)\n",
           deviceProp.maxTexture1D   , deviceProp.maxTexture2D[0], deviceProp.maxTexture2D[1],
           deviceProp.maxTexture3D[0], deviceProp.maxTexture3D[1], deviceProp.maxTexture3D[2]);
    mexPrintf("  Maximum Layered 1D Texture Size, (num) layers  1D=(%d), %d layers\n",
           deviceProp.maxTexture1DLayered[0], deviceProp.maxTexture1DLayered[1]);
    mexPrintf("  Maximum Layered 2D Texture Size, (num) layers  2D=(%d, %d), %d layers\n",
           deviceProp.maxTexture2DLayered[0], deviceProp.maxTexture2DLayered[1], deviceProp.maxTexture2DLayered[2]);
    
    
    mexPrintf("  Total amount of constant memory:               %lu bytes\n", deviceProp.totalConstMem);
    mexPrintf("  Total amount of shared memory per block:       %lu bytes\n", deviceProp.sharedMemPerBlock);
    mexPrintf("  Total number of registers available per block: %d\n", deviceProp.regsPerBlock);
    mexPrintf("  Warp size:                                     %d\n", deviceProp.warpSize);
    mexPrintf("  Maximum number of threads per multiprocessor:  %d\n", deviceProp.maxThreadsPerMultiProcessor);
    mexPrintf("  Maximum number of threads per block:           %d\n", deviceProp.maxThreadsPerBlock);
    mexPrintf("  Max dimension size of a thread block (x,y,z): (%d, %d, %d)\n",
           deviceProp.maxThreadsDim[0],
           deviceProp.maxThreadsDim[1],
           deviceProp.maxThreadsDim[2]);
    mexPrintf("  Max dimension size of a grid size    (x,y,z): (%d, %d, %d)\n",
           deviceProp.maxGridSize[0],
           deviceProp.maxGridSize[1],
           deviceProp.maxGridSize[2]);
    mexPrintf("  Maximum memory pitch:                          %lu bytes\n", deviceProp.memPitch);
    mexPrintf("  Texture alignment:                             %lu bytes\n", deviceProp.textureAlignment);
    mexPrintf("  Concurrent copy and kernel execution:          %s with %d copy engine(s)\n", (deviceProp.deviceOverlap ? "Yes" : "No"), deviceProp.asyncEngineCount);
    mexPrintf("  Run time limit on kernels:                     %s\n", deviceProp.kernelExecTimeoutEnabled ? "Yes" : "No");
    mexPrintf("  Integrated GPU sharing Host Memory:            %s\n", deviceProp.integrated ? "Yes" : "No");
    mexPrintf("  Support host page-locked memory mapping:       %s\n", deviceProp.canMapHostMemory ? "Yes" : "No");
    mexPrintf("  Alignment requirement for Surfaces:            %s\n", deviceProp.surfaceAlignment ? "Yes" : "No");
    mexPrintf("  Device has ECC support:                        %s\n", deviceProp.ECCEnabled ? "Enabled" : "Disabled");
    #if defined(WIN32) || defined(_WIN32) || defined(WIN64) || defined(_WIN64)
    mexPrintf("  CUDA Device Driver Mode (TCC or WDDM):         %s\n", deviceProp.tccDriver ? "TCC (Tesla Compute Cluster Driver)" : "WDDM (Windows Display Driver Model)");
    #endif
    mexPrintf("  Device supports Unified Addressing (UVA):      %s\n", deviceProp.unifiedAddressing ? "Yes" : "No");
    mexPrintf("  Device PCI Domain ID / Bus ID / location ID:   %d / %d / %d\n", deviceProp.pciDomainID, deviceProp.pciBusID, deviceProp.pciDeviceID);
    
  }
  else{
    mexPrintf("CUDA Device Query...\n");
    mexPrintf("There are %d CUDA devices.\n", devCount); 
  } 

}