#include <iostream>
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"

template <typename T>
__host__ __device__
inline T conj(const T& z){
  return z;
}

template <typename T>
struct cmath_functor : public thrust::unary_function< T , T >
{
  __host__ __device__
  T operator()( T x) const
  {
    return conj( x );
  }
};


template <typename T> void mx_mThrust_funmath_gen(const mxArray *prhs[]){
  thrust::device_vector<T> &ga = get_object< thrust::device_vector<T> >(mxGetProperty(prhs[0],0,"handle"));
  thrust::device_vector<T> &gb = get_object< thrust::device_vector<T> >(mxGetProperty(prhs[1],0,"handle"));
  
  assert(gb.size()==ga.size());
  thrust::transform(ga.begin(),ga.end(),gb.begin(), cmath_functor<T>());
}


template  <typename T>
void mx_mThrust_mathfun_gen_type(const mxArray *prhs[]){
  char *container[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"container")),
    mxArrayToString(mxGetProperty(prhs[1],0,"container"))
  };
  assert( strcmp(container[0],container[1])==0 );
  bool iscomplex[2];
  for (int i=0;i<2;i++){
    logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[i],0,"complex"));
    iscomplex[i]=*iscomp;
  }
  assert( iscomplex[0]==iscomplex[1] );
  
  if (strcmp(container[0], "device")==0){
    if (iscomplex[0]){
      mx_mThrust_funmath_gen<thrust::complex<T> >(prhs);
    }else 
      mx_mThrust_funmath_gen<T>(prhs); 
  }else if (strcmp(container[0], "host")==0){
    if (iscomplex[0])
      mx_mThrust_funmath_gen<thrust::complex<T> >(prhs);
    else 
      mx_mThrust_funmath_gen<T>(prhs);  
  }else
    mexErrMsgIdAndTxt("mThrustToolbox:subsgen","Unknow container."); 
}

// prhs[0] : handle to input mThrust object
// prhs[2] : handle to output mThrust object
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 2) 
    mexErrMsgTxt("Bad input... To do");
  
  assert( mxIsClass(prhs[0],"mThrust") && mxIsClass(prhs[1],"mThrust") );
  
  char *type[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"type")),
    mxArrayToString(mxGetProperty(prhs[1],0,"type"))
  };
  assert( strcmp(type[0],type[1])==0 );
  
  if (strcmp(type[0], "double")==0)
    mx_mThrust_mathfun_gen_type<double>(prhs);
  else if (strcmp(type[0], "float")==0)
    mx_mThrust_mathfun_gen_type<float>(prhs);
  else{
    mxFree(type[0]);mxFree(type[1]);
    mexErrMsgIdAndTxt("mThrustToolbox:mathfun_gen","type not implemented.");
  }
  mxFree(type[0]);mxFree(type[1]);
  MEXPRINTF("ending");
}