#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"

template <typename VectorIn,typename VectorOut> 
void mx_mThrust_copy(const mxArray *prhs[],
                     int istart, int iend, int ostart
){
  VectorIn &Vi = get_object< VectorIn >(mxGetProperty(prhs[0],0,"handle"));
  VectorOut &Vo = get_object< VectorOut >(mxGetProperty(prhs[1],0,"handle"));
  
  thrust::copy(Vi.begin()+istart,Vi.begin()+iend,Vo.begin()+ostart);
}

template <typename T> 
void mx_mThrust_copy_type(const mxArray *prhs[],
                          int istart, int iend, int ostart){
  char *icontainer =mxArrayToString(mxGetProperty(prhs[0],0,"container"));
  char *ocontainer =mxArrayToString(mxGetProperty(prhs[1],0,"container"));
  logical *iscomp0=(logical *)mxGetPr(mxGetProperty(prhs[0],0,"complex"));
  logical *iscomp1=(logical *)mxGetPr(mxGetProperty(prhs[1],0,"complex"));
  if (*iscomp0 != *iscomp1)
    mexErrMsgIdAndTxt("mThrustToolbox:copy","Trouble with complex/real");
  typedef typename thrust::host_vector<thrust::complex<T> > hVecCplx;
  typedef typename thrust::host_vector<T> hVecReal;
  typedef typename thrust::device_vector<thrust::complex<T> > dVecCplx;
  typedef typename thrust::device_vector<T> dVecReal;
  if (strcmp(icontainer, "host")==0){
    if (strcmp(ocontainer, "host")==0){
      if (*iscomp0==1)
        mx_mThrust_copy<hVecCplx,hVecCplx>(prhs,istart,iend,ostart);
      else
        mx_mThrust_copy<hVecReal,hVecReal>(prhs,istart,iend,ostart);
    }else if (strcmp(ocontainer, "device")==0){
      if (*iscomp0==1)
        mx_mThrust_copy<hVecCplx,dVecCplx>(prhs,istart,iend,ostart);
      else
        mx_mThrust_copy<hVecReal,dVecReal>(prhs,istart,iend,ostart);
    }else
      mexErrMsgIdAndTxt("mThrustToolbox:copy","container not implemented.");
  }else if (strcmp(icontainer, "device")==0){
    if (strcmp(ocontainer, "host")==0){
      if (*iscomp0==1)
        mx_mThrust_copy<dVecCplx,hVecCplx>(prhs,istart,iend,ostart);
      else
        mx_mThrust_copy<dVecReal,hVecReal>(prhs,istart,iend,ostart);
    }else if (strcmp(ocontainer, "device")==0){
      if (*iscomp0==1)
        mx_mThrust_copy<dVecCplx,dVecCplx>(prhs,istart,iend,ostart);
      else
        mx_mThrust_copy<dVecReal,dVecReal>(prhs,istart,iend,ostart);
    }else
      mexErrMsgIdAndTxt("mThrustToolbox:copy","container not implemented.");
  }else
    mexErrMsgIdAndTxt("mThrustToolbox:copy","container not implemented.");  
}


// prhs[0] : input Thrust object 
// prhs[1] : output Thrust object (same type  as input)
// prhs[2] : int scalar istart index (>=0) and (<dim)
// prhs[3] : int scalar iend index (>=0) and (<dim)
// prhs[4] : int scalar ostart index (>=0) and (<dim)
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 5) 
    mexErrMsgTxt("Bad input.");
  assert(mxIsClass(prhs[0],"mThrust") && mxIsClass(prhs[1],"mThrust"));
  
  if( mxGetNumberOfElements(prhs[2])!=1 )
    mexErrMsgIdAndTxt("mThrust:copy","3rd input value must be a scalar.");
  if( mxGetNumberOfElements(prhs[3])!=1 )
    mexErrMsgIdAndTxt("mThrust:copy","4th input value must be a scalar.");
  if( mxGetNumberOfElements(prhs[4])!=1 )
    mexErrMsgIdAndTxt("mThrust:copy","5th input value must be a scalar.");
  int *pistart = (int *) mxGetPr(prhs[2]);
  int *piend = (int *) mxGetPr(prhs[3]);
  int *postart = (int *) mxGetPr(prhs[4]);
  
  char *type =mxArrayToString(mxGetProperty(prhs[0],0,"type"));
  if (strcmp(type, "int")==0){
    mx_mThrust_copy_type<int>(prhs,*pistart,*piend,*postart);
  }else if (strcmp(type, "unsigned")==0){
    mx_mThrust_copy_type<unsigned>(prhs,*pistart,*piend,*postart);
  }else if (strcmp(type, "float")==0){
    mx_mThrust_copy_type<float>(prhs,*pistart,*piend,*postart);
  }else if (strcmp(type, "double")==0){
    mx_mThrust_copy_type<double>(prhs,*pistart,*piend,*postart);
  }else  if (strcmp(type, "logical")==0){
    mx_mThrust_copy_type<logical>(prhs,*pistart,*piend,*postart);
  }else{
    mxFree(type);
    mexErrMsgIdAndTxt("mThrustToolbox:copy","type not implemented.");
  }
  MEXPRINTF("ending");
}