#include <iostream>
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"
#include <thrust/device_ptr.h>
#include <thrust/sequence.h>

template <typename T> 
struct disreg : public thrust::unary_function<int,T>
{
  const thrust::complex<T> a, h;
  
  disreg(thrust::complex<T> _a , thrust::complex<T> _h) : a(_a), h(_h) {};
  
  __host__ __device__
  thrust::complex<T> operator()(const int& i) const{
    return a+T(i)*h;
  }
};

template <typename T,typename Vector> void  mThrust_linspace_complex(const mxArray *prhs[]){
  
  Vector &V = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  T *pfirst_r=(T *)mxGetPr(prhs[1]);
  T *pfirst_i=(T *)mxGetPi(prhs[1]);
  T *pstep_r=(T *)mxGetPr(prhs[2]);
  T *pstep_i=(T *)mxGetPi(prhs[2]);
  thrust::complex<T> first(*pfirst_r,*pfirst_i);
  thrust::complex<T> step(*pstep_r,*pstep_i);
  //thrust::sequence(V.begin(),V.end(),first,step);
  
  thrust::transform(thrust::counting_iterator<unsigned int>(0),
                    thrust::counting_iterator<unsigned int>(V.size()),
                    V.begin(),
                    disreg<T>(first,step));
}

template <typename T,typename Vector> void  mThrust_linspace_real(const mxArray *prhs[]){
 
  Vector &V = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  T *pfirst=(T *)mxGetPr(prhs[1]);
  T *pstep=(T *)mxGetPr(prhs[2]);
  thrust::sequence(V.begin(),V.end(),*pfirst,*pstep);
}

template <typename T>
void  mThrust_linspace_type(const mxArray *prhs[]){
  char *container =mxArrayToString(mxGetProperty(prhs[0],0,"container"));
  logical *pisComplex=(logical *) mxGetPr(mxGetProperty(prhs[0],0,"complex"));
  
  // Add test prhs[1] and prhs[2] are complex if *pisComplex
  
  if (strcmp(container, "host")==0){
    if (*pisComplex)
      mThrust_linspace_complex<T,thrust::host_vector<thrust::complex<T> > >(prhs);
    else
      mThrust_linspace_real<T,thrust::host_vector<T> >(prhs);
  }else if (strcmp(container, "device")==0){
    if (*pisComplex)
      mThrust_linspace_complex<T,thrust::device_vector<thrust::complex<T> > >(prhs);
    else
      mThrust_linspace_real<T,thrust::device_vector<T> >(prhs);
  }else
    mexWarnMsgIdAndTxt("mThrustToolbox:mThrust_linspace","Container failed.");
}

// prhs[0] : output mThrust object (not complex)
// prhs[1] : start value
// prhs[2] : step 
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 3) 
    mexErrMsgTxt("Bad input.");
  
  assert(mxIsClass(prhs[0],"mThrust"));
  char *type =mxArrayToString(mxGetProperty(prhs[0],0,"type"));
  if (strcmp(type, "double")==0){
    assert( (mxGetClassID(prhs[1])==mxDOUBLE_CLASS) && (mxGetClassID(prhs[2])==mxDOUBLE_CLASS) );
    mThrust_linspace_type<double>(prhs);
  }else if (strcmp(type, "float")==0){
    assert( (mxGetClassID(prhs[1])==mxSINGLE_CLASS) && (mxGetClassID(prhs[2])==mxSINGLE_CLASS) );
    mThrust_linspace_type<float>(prhs);
  }else
    mexWarnMsgIdAndTxt("mThrustToolbox:mThrust_linspace","Type failed.");
  mxFree(type);
  MEXPRINTF("ending");
}  