#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"
#include <time.h>

#include <thrust/sort.h>
#include <thrust/host_vector.h>
#include <thrust/device_vector.h>

#define MESS "mThrustToolbox:sort_by_key"

template <typename VecKey,typename VecVal> void  mThrust_sort_by_key(const mxArray *prhs[]){
  VecKey &keys = get_object< VecKey >( mxGetProperty(prhs[0],0,"handle") );
  VecVal &vals = get_object< VecVal >( mxGetProperty(prhs[1],0,"handle") );
  unsigned *pFlag = (unsigned *)mxGetData(prhs[2]);
  typedef typename VecKey::value_type T;
  if (*pFlag==0)
    thrust::sort_by_key(keys.begin(),keys.end(),vals.begin(),thrust::greater<T>());
  else
    thrust::sort_by_key(keys.begin(),keys.end(),vals.begin(),thrust::less<T>());
}

template <typename Tkey, typename Tval> void  mThrust_sort_by_key_container(const mxArray *prhs[]){
  char *container[2] = {
          mxArrayToString(mxGetProperty(prhs[0],0,"container")),
          mxArrayToString(mxGetProperty(prhs[1],0,"container"))
  };
  assert( strcmp(container[0],container[1])==0 );
  if (strcmp(container[0], "host")==0){
    typedef thrust::host_vector<Tkey> VecKey;
    typedef thrust::host_vector<Tval> VecVal;
    mThrust_sort_by_key<VecKey,VecVal>(prhs);
  }else if (strcmp(container[0], "device")==0){
    typedef thrust::device_vector<Tkey> VecKey;
    typedef thrust::device_vector<Tval> VecVal;
    mThrust_sort_by_key<VecKey,VecVal>(prhs);
  }else mexWarnMsgIdAndTxt(MESS,"container '%s' not allowed.",container[0]);
  
  mxFree(container[0]);mxFree(container[1]);
}

template <typename Tkey> void  mThrust_sort_by_key_type1(const mxArray *prhs[]){
  char *type=mxArrayToString(mxGetProperty(prhs[1],0,"type"));
  if (strcmp(type, "float")==0)
    mThrust_sort_by_key_container<Tkey,float>(prhs);
  else if (strcmp(type, "double")==0)
    mThrust_sort_by_key_container<Tkey,double>(prhs);
  else if (strcmp(type, "int")==0)
    mThrust_sort_by_key_container<Tkey,int>(prhs);
  else if (strcmp(type, "unsigned")==0)
    mThrust_sort_by_key_container<Tkey,unsigned>(prhs);
  else if (strcmp(type, "logical")==0)
    mThrust_sort_by_key_container<Tkey,logical>(prhs);
  else
    mexWarnMsgIdAndTxt(MESS,"sort_by_key of mThrust<%s> not allowed.",type);
  mxFree(type);
}
  
void  mThrust_sort_by_key_type0(const mxArray *prhs[]){
  char *type=mxArrayToString(mxGetProperty(prhs[0],0,"type"));
  if (strcmp(type, "float")==0)
    mThrust_sort_by_key_type1<float>(prhs);
  else if (strcmp(type, "double")==0)
    mThrust_sort_by_key_type1<double>(prhs);
  else if (strcmp(type, "int")==0)
    mThrust_sort_by_key_type1<int>(prhs);
  else if (strcmp(type, "unsigned")==0)
    mThrust_sort_by_key_type1<unsigned>(prhs);
  else if (strcmp(type, "logical")==0)
    mThrust_sort_by_key_type1<logical>(prhs);
  else
    mexWarnMsgIdAndTxt(MESS,"sort_by_key of mThrust<%s> not allowed.",type);
  mxFree(type);
}
  
// prhs[0] -> keys, mThrust vector
// prhs[1] -> index, mThrust vector same container and dimension as keys
// prhs[2] -> double, 0 for ascending order otherwise descending order
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 3) 
    mexErrMsgTxt("Bad input.");
  assert(mxIsClass(prhs[0],"mThrust") && mxIsClass(prhs[1],"mThrust") && (mxGetClassID(prhs[2])==mxUINT32_CLASS));
  mThrust_sort_by_key_type0(prhs);
  
  MEXPRINTF("ending");
}