/*
*/
#include <iostream>
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"
 
template <typename T> __global__ void subsref_kernel(
    T *d_a,
    int N,
    int *d_s,
    int n,
    T *d_b
){
    const int      tid = blockDim.x * blockIdx.x + threadIdx.x;
    const int THREAD_N = blockDim.x * gridDim.x;
    int pos=tid;
    
    while (pos < n){
       d_b[pos]=d_a[d_s[pos]];
      pos+=THREAD_N;
    }
}

template <typename T> void mx_mThrust_subsgen_device(const mxArray *prhs[]){
  thrust::device_vector<T> &ga = get_object< thrust::device_vector<T> >(mxGetProperty(prhs[0],0,"handle"));
  thrust::device_vector<T> &gb = get_object< thrust::device_vector<T> >(mxGetProperty(prhs[2],0,"handle"));
  thrust::device_vector<int> &gs = get_object< thrust::device_vector<int> >(mxGetProperty(prhs[1],0,"handle"));
  
  assert(gb.size()==gs.size());
  T *da=thrust::raw_pointer_cast(&ga[0]);
  T *db=thrust::raw_pointer_cast(&gb[0]);
  int *ds=thrust::raw_pointer_cast(&gs[0]);
  subsref_kernel<T><<<256,256>>>(da,ga.size(),ds,gs.size(),db);
}

template <typename T> void mx_mThrust_subsgen_host(const mxArray *prhs[]){
  thrust::host_vector<T> &ga = get_object< thrust::host_vector<T> >(mxGetProperty(prhs[0],0,"handle"));
  thrust::host_vector<T> &gb = get_object< thrust::host_vector<T> >(mxGetProperty(prhs[2],0,"handle"));
  thrust::host_vector<int> &gs = get_object< thrust::host_vector<int> >(mxGetProperty(prhs[1],0,"handle"));
  
  assert(gb.size()==gs.size());
  for (int i=0;i<gb.size();i++)
    gb[i]=ga[gs[i]];
  
}

#include "mex_mThrust_subsgen.cu"