#include <iostream>
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"
#include <thrust/unique.h>

#define MESS "mThrustToolbox:unique_copy"

template <typename Vector>
void mx_mThrust_unique(const mxArray *prhs[])
{
  typename Vector::iterator B_end;
  Vector &A = get_object<Vector>(mxGetProperty(prhs[0],0,"handle"));
  Vector &B = get_object<Vector>(mxGetProperty(prhs[1],0,"handle"));
  B_end = thrust::unique_copy(A.begin(), A.end(), B.begin());
  // shrink B to exactly fit output
  B.erase(B_end, B.end());
}

template  <typename T>
void mx_mThrust_unique_type(const mxArray *prhs[]){
  char *container[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"container")),
    mxArrayToString(mxGetProperty(prhs[1],0,"container"))
  };
  if ( strcmp(container[0],container[1])!=0 ){
    mexWarnMsgIdAndTxt(MESS,"The mThrust vectors must have the same container.");
    return;
  }
  bool iscomplex[2];
  for (int i=0;i<2;i++){
    logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[i],0,"complex"));
    iscomplex[i]=*iscomp;
  }
  if ( iscomplex[1]!=iscomplex[0] ){
    mexWarnMsgIdAndTxt(MESS,"The mThrust vectors must be both complex or real.");
    return;
  }
  if (strcmp(container[0], "device")==0){
    typedef thrust::device_vector<thrust::complex<T> > VecCplx;
    typedef thrust::device_vector<T> VecReal;
    if (iscomplex[0])
      mx_mThrust_unique<VecCplx>(prhs); 
    else
      mx_mThrust_unique<VecReal>(prhs);
  }else if (strcmp(container[0], "host")==0){
    typedef thrust::host_vector<thrust::complex<T> > VecCplx;
    typedef thrust::host_vector<T> VecReal;
    if (iscomplex[0])
      mx_mThrust_unique<VecCplx>(prhs); 
    else
      mx_mThrust_unique<VecReal>(prhs);
  }else 
    mexWarnMsgIdAndTxt(MESS,"Unknow container %s.",container[0]); 
  mxFree(container[0]);mxFree(container[1]);
}

// prhs[0] mThrust object to find zeros
// prhs[1] mThrust <int> object (same container as prhs[0])
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 2) 
    mexErrMsgTxt("Bad input... To do");
  
  assert(mxIsClass(prhs[0],"mThrust") && mxIsClass(prhs[1],"mThrust") );
  char *type[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"type")),
    mxArrayToString(mxGetProperty(prhs[1],0,"type"))
  };
  if ( strcmp(type[1],type[0])!=0 ){
    mexWarnMsgIdAndTxt(MESS,"The mThrust vectors must have the same 'type'.");
    mxFree(type[0]);mxFree(type[1]);
    return;
  }
  
  if (strcmp(type[0], "double")==0)
    mx_mThrust_unique_type<double>(prhs);
  else if (strcmp(type[0], "float")==0)
    mx_mThrust_unique_type<float>(prhs);
  else if (strcmp(type[0], "int")==0)
    mx_mThrust_unique_type<int>(prhs);
  else if (strcmp(type[0], "unsigned")==0)
    mx_mThrust_unique_type<unsigned>(prhs);
  else
    mexWarnMsgIdAndTxt(MESS,"Find of mThrust<%s> not allowed.",type);
  mxFree(type[0]);mxFree(type[1]);
}